"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmartFeed = void 0;
const rest_client_1 = require("./rest-client");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const util_1 = require("./util");
// t = voltage / 32767 * 3.6 * 2
// t >= 5.9: 3/3, t >= 5.5: 2/3, t >= 5: 1/3, t >= 0: 0/3
const minVoltage = 22755, maxVoltage = 29100;
function getBatteryLevel(info) {
    const voltage = +info.battery_voltage;
    if (!info.is_batteries_installed ||
        isNaN(voltage) ||
        voltage < 100 ||
        voltage > maxVoltage) {
        return 100;
    }
    return Math.max((100 * (voltage - minVoltage)) / (maxVoltage - minVoltage), 0);
}
class SmartFeed {
    constructor(initialInfo, restClient) {
        this.initialInfo = initialInfo;
        this.restClient = restClient;
        this.onInfoRequested = new rxjs_1.Subject();
        this.onState = new rxjs_1.BehaviorSubject(this.initialInfo);
        this.onBatteryLevel = this.onState.pipe((0, operators_1.map)(getBatteryLevel), (0, operators_1.distinctUntilChanged)());
        (0, rxjs_1.merge)(this.onInfoRequested.pipe((0, operators_1.throttleTime)(2000)), // fire immediately
        this.onInfoRequested.pipe((0, operators_1.debounceTime)(2000)) // fire again after 2 seconds of quiet
        ).subscribe(() => this.fetchInfo().catch((e) => {
            (0, util_1.logError)(`Failed to fetch info for ${this.name}`);
            (0, util_1.logError)(e);
        }));
    }
    get id() {
        return this.initialInfo.id;
    }
    get name() {
        return this.state.settings.friendly_name;
    }
    get state() {
        return this.onState.getValue();
    }
    getBatteryLevel() {
        return getBatteryLevel(this.state);
    }
    feederPath(path = '') {
        return (0, rest_client_1.apiPath)(`feeders/${this.initialInfo.thing_name}${path ? '/' + path : ''}`);
    }
    feed({ amount = 1, slowFeed = this.state.settings.slow_feed } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            (0, util_1.logInfo)(`Feeding ${this.name} ${amount}/8 Cups ${slowFeed ? 'Slowly' : ''}`);
            yield this.restClient.request({
                method: 'POST',
                url: this.feederPath('meals'),
                json: {
                    amount,
                    slow_feed: slowFeed,
                },
            });
            this.requestInfoUpdate();
        });
    }
    getHistory({ days = 15 } = {}) {
        return this.restClient.request({
            method: 'GET',
            url: this.feederPath(`messages?days=${days}`),
        });
    }
    getLastFeedingMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            const messages = yield this.getHistory();
            for (const message of messages) {
                if (message.message_type === 'FEED_DONE') {
                    return message;
                }
            }
        });
    }
    repeatLastFeed() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const lastFeeding = yield this.getLastFeedingMessage(), amount = ((_a = lastFeeding === null || lastFeeding === void 0 ? void 0 : lastFeeding.payload) === null || _a === void 0 ? void 0 : _a.amount) || 1;
            yield this.feed({ amount });
        });
    }
    fetchInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const info = yield this.restClient.request({
                method: 'GET',
                url: this.feederPath(),
            });
            this.onState.next(info);
            return info;
        });
    }
    requestInfoUpdate() {
        this.onInfoRequested.next(null);
    }
}
exports.SmartFeed = SmartFeed;
